﻿namespace BookCatalog.Attributes.Validation
{
    using System;
    using System.ComponentModel.DataAnnotations;

    [AttributeUsage(AttributeTargets.Property, AllowMultiple = false, Inherited = true)]
    public class ValidDateAttribute : ValidationAttribute
    {
        private DateTime _minimumDate;
        private string _minimumDateString;

        public ValidDateAttribute()
            : base("Invalid date value.")
        {
        }

        public string MinDate
        {
            get { return this._minimumDateString; }
            set
            {
                if (!DateTime.TryParse(value, out this._minimumDate)) {
                    throw new ArgumentException("Invalid MinimumDate value.");
                }

                this._minimumDateString = value;
            }
        }

        public string MinDateErrorMessage { get; set; }

        public string MinDateErrorMessageResourceName { get; set; }

        #region ValidationAttribute overrides

        public override string FormatErrorMessage(string name)
        {
            return string.Format(
                this.ErrorMessageString, name, this._minimumDateString);
        }

        protected override ValidationResult IsValid(object value, ValidationContext validationContext)
        {
            if (!(value is DateTime)) {
                throw new ArgumentException(
                    string.Format(
                        "{0} property type is not DataTime. [ValidDate] attribute must be used with DataTime properties only.",
                        validationContext.DisplayName));
            }

            var enteredDate = (DateTime)value;

            if (enteredDate < this._minimumDate) {
                return new ValidationResult(
                    this.GetMinDateErrorMessage(validationContext.DisplayName));
            }

            if (enteredDate > DateTime.Now) {
                return new ValidationResult(
                    this.FormatErrorMessage(validationContext.DisplayName));
            }

            return ValidationResult.Success;
        }

        #endregion

        private string GetMinDateErrorMessage(string name)
        {
            if (this.ErrorMessageResourceType == null) {
                return this.MinDateErrorMessage;
            }

            var errorMessageProperty =
                this.ErrorMessageResourceType.GetProperty(this.MinDateErrorMessageResourceName);

            var errorMessage = (string)errorMessageProperty.GetValue(null, null);

            return string.Format(errorMessage, name, this._minimumDateString);
        }
    }
}